==> lagrange-1.3.0.tar.gz: lagrange-1.3.0/src/periodic.h <==
/* Copyright 2021 Jaakko Keränen <jaakko.keranen@iki.fi>

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

#include <the_Foundation/mutex.h>
#include <the_Foundation/sortedarray.h>

iDeclareType(Periodic)
iDeclareType(Thread)

/* Animation utility. Not per frame but several times per second. Thread safe. */
struct Impl_Periodic {
    iMutex *     mutex;
    iSortedArray commands;
    uint32_t     lastPostTime;
};

void    init_Periodic   (iPeriodic *);
void    deinit_Periodic (iPeriodic *);

iLocalDef iBool isEmpty_Periodic(const iPeriodic *d) {
    return isEmpty_SortedArray(&d->commands);
}

void    add_Periodic            (iPeriodic *, iAny *context, const char *command);
void    remove_Periodic         (iPeriodic *, iAny *context);

iBool   postCommands_Periodic   (iPeriodic *);

==> lagrange-1.4.2.tar.gz: lagrange-1.4.2/src/periodic.h <==
/* Copyright 2021 Jaakko Keränen <jaakko.keranen@iki.fi>

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

#pragma once

#include <the_Foundation/mutex.h>
#include <the_Foundation/ptrset.h>
#include <the_Foundation/sortedarray.h>

iDeclareType(Periodic)
iDeclareType(Thread)

/* Animation utility. Not per frame but several times per second. Thread safe. */
struct Impl_Periodic {
    iMutex *     mutex;
    iSortedArray commands;
    uint32_t     lastPostTime;
    iPtrSet      pendingRemoval; /* contexts */
};

void    init_Periodic   (iPeriodic *);
void    deinit_Periodic (iPeriodic *);

iLocalDef iBool isEmpty_Periodic(const iPeriodic *d) {
    return isEmpty_SortedArray(&d->commands);
}

void    add_Periodic            (iPeriodic *, iAnyObject *context, const char *command);
void    remove_Periodic         (iPeriodic *, iAnyObject *context);

iBool   dispatchCommands_Periodic( iPeriodic *);
